// Dart imports:
import 'dart:math';

// Flutter imports:
import 'package:flutter/material.dart';

// Package imports:
import 'package:cached_network_image/cached_network_image.dart';

// Project imports:
import 'package:zego_uikit/src/components/audio_video/avatar/ripple_avatar.dart';
import 'package:zego_uikit/src/services/internal/internal.dart';
import 'package:zego_uikit/zego_uikit.dart';

/// @nodoc
class ZegoAvatar extends StatelessWidget {
  final Size avatarSize;
  final ZegoUIKitUser? user;
  final bool showAvatar;
  final bool showSoundLevel;
  final Size? soundLevelSize;
  final Color? soundLevelColor;
  final ZegoAvatarBuilder? avatarBuilder;

  const ZegoAvatar({
    Key? key,
    required this.avatarSize,
    this.user,
    this.showAvatar = true,
    this.showSoundLevel = false,
    this.soundLevelSize,
    this.soundLevelColor,
    this.avatarBuilder,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    if (!showAvatar || user == null) {
      return Container(color: Colors.transparent);
    }

    final centralAvatar = avatarBuilderWithURL(context, avatarSize, user, {});
    return Center(
      child: SizedBox.fromSize(
        size: showSoundLevel ? soundLevelSize : avatarSize,
        child: showSoundLevel
            ? ZegoRippleAvatar(
                user: user,
                color: soundLevelColor ?? const Color(0xff6a6b6f),
                minRadius: min(avatarSize.width, avatarSize.height) / 2,
                radiusIncrement: 0.06,
                child: centralAvatar,
              )
            : centralAvatar,
      ),
    );
  }

  Widget avatarBuilderWithURL(
    BuildContext context,
    Size size,
    ZegoUIKitUser? user,
    Map<String, dynamic> extraInfo,
  ) {
    return ValueListenableBuilder(
      valueListenable: ZegoUIKitUserPropertiesNotifier(
        user ?? ZegoUIKitUser.empty(),
      ),
      builder: (context, _, __) {
        final avatarURL = user?.inRoomAttributes.value.avatarURL ?? '';
        return avatarBuilder?.call(
              context,
              size,
              user,
              extraInfo,
            ) ??
            (avatarURL.isNotEmpty
                ? CachedNetworkImage(
                    imageUrl: avatarURL,
                    imageBuilder: (context, imageProvider) => Container(
                      decoration: BoxDecoration(
                        shape: BoxShape.circle,
                        image: DecorationImage(
                          image: imageProvider,
                          fit: BoxFit.cover,
                        ),
                      ),
                    ),
                    progressIndicatorBuilder:
                        (context, url, downloadProgress) =>
                            CircularProgressIndicator(
                                value: downloadProgress.progress),
                    errorWidget: (context, url, error) {
                      ZegoLoggerService.logInfo(
                        '$user avatar url is invalid',
                        tag: 'uikit',
                        subTag: 'avatar',
                      );
                      return circleName(context, avatarSize, user);
                    },
                  )
                : circleName(context, avatarSize, user));
      },
    );
  }

  Widget circleName(BuildContext context, Size size, ZegoUIKitUser? user) {
    final userName = user?.name ?? '';
    return SizedBox.fromSize(
      size: size,
      child: Container(
        decoration: const BoxDecoration(
          color: Color(0xffDBDDE3),
          shape: BoxShape.circle,
        ),
        child: Center(
          child: Text(
            userName.isNotEmpty ? userName.characters.first : '',
            style: TextStyle(
              fontSize: showSoundLevel ? size.width / 4 : size.width / 5 * 4,
              color: const Color(0xff222222),
              decoration: TextDecoration.none,
            ),
          ),
        ),
      ),
    );
  }
}
